#include <lx_string.h>
#include <get_opts.h>
#include "opt_misc.h"
#include "defines.h"
#include "sockdata.h"
#include "winch.h"

/* usage: $0 [options] <-f sockfile> <out-sock>  */

static int fail_switch ();

add_opt (opt_a, "a", "abort-on-fail", OPT_BOOL,     (opt_bool_t)0, fail_switch);
/* simply remove failed sockets from the monitoring list */
add_opt (opt_i, "i", "ignore-fail",   OPT_BOOL,     (opt_bool_t)0, fail_switch);
/* --retry-fail millisecs */
add_opt (opt_r, "r", "retry-fail",    OPT_ULONG,   (opt_ulong_t)1000, fail_switch);
/* ... and if we can't open the output socket... */
add_opt (opt_R, "R", "retry-failout", OPT_ULONG,   (opt_ulong_t)1000, 0);
/* how many times to retry the output socket on fail */
add_opt (opt_N, "N", "n-retries-out", OPT_ULONG,   (opt_ulong_t)0, 0);
add_opt (opt_f, "f", "socklist-file", OPT_STRING, (opt_string_t)0, 0);
add_opt (opt_b, "b", "bind-socks",    OPT_BOOL,     (opt_bool_t)0, 0);
add_opt (opt_B, "B", "bind-outsock",  OPT_BOOL,     (opt_bool_t)0, 0);
/* default socket types are SOCK_STREAM.  options are for SOCK_DGRAM. */
add_opt (opt_d, "d", "sock-dgram-in", OPT_BOOL,     (opt_bool_t)0, 0);
add_opt (opt_D, "D", "sock-dgram-out",OPT_BOOL,     (opt_bool_t)0, 0);

put_opts (options, &opt_a, &opt_i, &opt_r, &opt_R, &opt_N, &opt_f,
		   &opt_b, &opt_B, &opt_d, &opt_D);



struct socketlink *sl_out = 0;
struct socketlink *sl_in  = 0;

int main (argc, argv)
    int argc;
    char **argv;
{
    char **args;
    int argn;

    argn = get_opts_errormatic (&args, argv+1, argc-1, options);

    if (!argn || !OPT_SOCKLIST_FILE) die_usage(argv[0]);

    socktype_setup();

    dprintf ("registering sockets...");
    register_socket      (&sl_out, args[0], SW_OUTPUT);
    register_socketfiles (&sl_in, OPT_SOCKLIST_FILE, SW_INPUT);
    dprintf (" done.\n");

    select_loop ();

    return 0;
}


static int fail_switch (opt *optp) {
        if (optp != &opt_a) opt_a.v_opt.opt_bool  = 0;
        if (optp != &opt_i) opt_i.v_opt.opt_bool  = 0;
        if (optp != &opt_r) opt_r.v_opt.opt_ulong = 0;
        return 0;
}
