#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <lx_string.h>
#include "defines.h"
#include "sockdata.h"
#include "winch.h"


static void
register_socketb (slp, name, method)
	struct socketlink **slp;
	lx_s *name;
	int method;
{
	struct socketlink *slnew;

	slnew = socketlink_register (slp, name);
	if (!slnew) dprintf("socket link already exists in chain\n");

	slnew->method = method;
}


void
register_socket (slp, name, method)
	struct socketlink **slp;
	char *name;
	int method;
{
	/* explicitly set sname; do not free. */
	lx_s sname = {name};
	sname.alloc = sname.len = strlen(name);

	register_socketb(slp, &sname, method);
}

void
register_socketfiles (slp, fname, method)
	struct socketlink **slp;
	char *fname;
	int method;
{
	int fd;
	lx_gd gd;
	lx_s s = {0};
	unsigned char match;

restart:
	fd = open (fname, O_RDONLY);

	if (fd == -1) {
		if (errno == EINTR) goto restart;
		die_error3("open(", fname, ")");
	}

	if (lx_gdnew (&gd, fd, 0)) die_nomem();

	do {
		/* file should not be hostile; unlimited length is okay */
		if (lx_getln (&s, &gd, &match, 0))
			die_error2("reading ", fname);

		/* don't need to check MATCH_TOOLONG; line is unlimited */
	
		lx_chomp_ws(&s);

		if (s.len) register_socketb(slp, &s, method);

		lx_free (&s);

	} while (LX_match_ok(match));

	lx_gdfree (&gd);
}
