#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <string.h>
#include <errno.h>
#include <lx_string.h>
#include "defines.h"
#include "sockdata.h"
#include "winch.h"


/***************************************************************************
  socket_accept (fd_set *set, struct socketlink **slp);

  Accepts new connection from sl->sd.
  Creates a new link for sd.
  Registers new link with chain, and adds new sd to set.
  Increases maxfd if necessary.

  returns the new link, or 0 on error.

 ***************************************************************************/

extern int maxfd;

void *
socket_accept(fd_set *set, struct socketlink *sl)
{
	struct socketlink *slnew;
	struct sockaddr addr;           /* ignored */
	int    addrsize = sizeof(addr); /* ignored */

	dprintf ("accepting new connection on STREAM socket (%d)\n", sl->sd);

	slnew = socketlink_new();
	if (!slnew) die_nomem();

	slnew->sd = accept(sl->sd, &addr, &addrsize);
	/* if EINTR, we'll get it next select */
	if (slnew->sd == -1) {
		dprintf ("no connection ready on descriptor %d (%s)\n",
				sl->sd, strerror(errno));
		socketlink_destroy(slnew);
		return 0;
	}

	if (lx_strcopy(&slnew->name, &sl->name))
		die_nomem();

	slnew->type = SOCK_ESTABLISHED;

	/* sl must be non-null, so *(&sl) will never be modified in
	 * socketlink_linkup() */
	if (socketlink_linkup(&sl, slnew)) {
		socketlink_destroy(slnew);
		return 0;
	}

	dprintf ("linked new connection for descriptor %d; adding to set.\n",
			slnew->sd);

	FD_SET(slnew->sd, set);
	if (slnew->sd > maxfd) maxfd = slnew->sd;

	return slnew;
}
