#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <lx_string.h>
#include "defines.h"
#include "sockdata.h"
#include "winch.h"

void *
socketlink_new(void)
{
	struct socketlink *sl;

	sl = malloc (sizeof (*sl));
	if (!sl) die_nomem();

	sl->name.s   = 0;
	sl->sd       = -1;
	sl->attempts = 0;
	sl->method   = 0;
	sl->type     = 0;
	sl->buf      = 0;
	sl->next     = 0;

	return sl;
}

int
socketlink_linkup(struct socketlink **slp, struct socketlink *slnew)
{
	struct socketlink *sl = *slp;
	struct socketlink *slprev = 0;

	for (;sl;) {
		if ((slnew->type != SOCK_ESTABLISHED) &&
				!lx_strcmp(&sl->name, &slnew->name))
			return 1;

		slprev = sl;
		sl = sl->next;
	}

	if (!slprev) *slp = slnew;
	else slprev->next = slnew;

	return 0;
}

void *
socketlink_register(struct socketlink **slp, lx_s *name)
{
	struct socketlink *sl;

	sl = socketlink_new();
	if (!sl) die_nomem();

	if (lx_strcopy(&sl->name, name))
		die_nomem();

	if (socketlink_linkup(slp, sl)) {
		/* link with this name already exists */
		socketlink_destroy(sl);
		return 0;
	}

	return sl;
}

void
socketlink_unregister(struct socketlink **slp, lx_s *name)
{
	struct socketlink *sl = *slp;
	struct socketlink *slprev = 0;

	for (;sl;) {
		if (!lx_strcmp(name, &sl->name)) {
			if (slprev) slprev->next = sl->next;
			else        *slp = sl->next;

			socketlink_close(sl);
			socketlink_destroy(sl);
			return;
		}

		slprev = sl;
		sl = sl->next;
	}

}

void
socketlink_destroy(struct socketlink *sl)
{
	if (sl->name.s) lx_free (&sl->name);
	if (sl->buf) free (sl->buf);
	memset (sl, 0, sizeof(struct socketlink));
	free(sl);
}

int
socketlink_close(struct socketlink *sl)
{
	int rv;
	do {
		rv = close (sl->sd);
		if (!rv || (errno != EINTR)) break;
	} while (1);

	return rv;
}

